<?php

namespace CloudPrinterPlugin\Controllers;

use CloudPrinter\CloudApps\Client\CloudAppsClient;
use CloudPrinter\CloudApps\Exception\ValidationException;
use CloudPrinterPlugin\Services\ConfigService;
use CloudPrinterPlugin\Services\RevenueCheckService;
use CloudPrinterPlugin\Services\ShippingService;
use CloudPrinter\CloudApps\Model\{CountryInterface, Option, OrderQuote, OrderQuoteItem};
use CloudPrinterPlugin\Helpers\{CartHelper, CloudPrinterHelper, LogHelper, OrderQuoteCacheHelper};
use CloudPrinterPlugin\Libs\BaseController;
use CloudPrinterPlugin\Models\CPProduct;
use CloudPrinterPlugin\Services\DynamicProduct\DynamicProductService;

/**
 * Class CheckoutController
 *
 * @copyright   2019 by CloudPrinter
 * @author      Vasyl Kushniruk <vk@cloudprinter.com>
 */
class CheckoutController extends BaseController
{
    public $isFreeShipping = false;
    public $costCheck = true;

    public function __construct()
    {
        $this->isFreeShipping = get_option('cp_free_shipping') === 'yes';
        $this->costCheck = get_option('cp_cost_check') == 'yes';
        parent::__construct(CP_ABS_PATH, 'checkout');

        // Display custom shipping on checkout page. For print product.
        add_action('woocommerce_review_order_before_order_total', [$this, 'displayShippingForPrintProduct']);
        add_action('woocommerce_cart_totals_before_order_total', [$this, 'displayShippingForPrintProduct']);

        // Custom validation for checkout page.
        add_action('woocommerce_before_checkout_process', [$this, 'checkoutValidate']);

        // Set label for stock shipping.
        add_filter('woocommerce_shipping_package_name', [$this, 'setShippingStockName']);

        // Enable stock shipping when stock products in cart.
        add_filter('woocommerce_cart_needs_shipping', [$this, 'stockShippingEnabled']);

        // Calculate total price on checkout page.
        add_filter('woocommerce_calculated_total', [$this, 'calculateTotalOrderPrice']);

        // Set required postcode
        add_filter('woocommerce_get_country_locale', [$this, 'changeCountryLocale']);

        // Display additional shipping address
        add_filter('woocommerce_cart_needs_shipping_address', [$this, 'needShippingAddress']);

        // Store order item key.
        add_action('woocommerce_checkout_create_order_line_item', [$this, 'checkoutCreateOrderLineItem'], 10, 4);

        // Endpoint for storing selected print shipping method.
        add_action('rest_api_init', function () {
            register_rest_route('cloudprinter/v1', '/cart/save-print-shipping', [
                'methods' => 'POST',
                'callback' => [$this, 'savePrintShipping'],
                'permission_callback' => '__return_true',
            ]);
        });

        // Set phone number as required
        add_filter('woocommerce_checkout_fields', [$this, 'overrideCheckoutFields']);

        // Save custom checkout fields
        add_action('woocommerce_checkout_update_order_meta', [$this, 'saveCustomCheckoutFields']);

        // Custom validation for checkout form
        add_action('woocommerce_after_checkout_validation', [$this, 'validationCheckoutForm'], 10, 2);

        // Clear the shipping quotes after add/remove product in cart
        add_action('woocommerce_cart_item_removed', [$this, 'clearShippingQuotes']);
        add_action('woocommerce_add_to_cart', [$this, 'clearShippingQuotes']);
        add_action('woocommerce_after_cart_item_quantity_update', [$this, 'clearShippingQuotes']);

        // Display the selected dynamic options on the cart page
        add_filter('woocommerce_get_item_data', [$this, 'displayCartItemDynamicOptions'], 10, 2);

        // Don`t display the print shipping error on cart overview page
        add_filter('cloudprinter_cart_review_display_shipping_error', '__return_false');

        add_action( 'woocommerce_before_order_notes', [$this, 'addCustomerIdentificationField'] );

        add_action( 'woocommerce_checkout_update_order_meta', [$this, 'saveCustomerIdentification'] );

        add_action( 'woocommerce_thankyou', [$this, 'displayCustomerIdentificationOnOrderThankyou'] );
    }

    public function addCustomerIdentificationField($checkout)
    {
        woocommerce_form_field( 'customer_identification', array(
            'type' => 'text',
            'class' => array( 'form-row-wide' ),
            'id' => 'customer_identification',
            'label' => 'Customer identification',
            'required' => false,
        ), $checkout->get_value( 'customer_identification' ) );
    }

    public function saveCustomerIdentification($orderId)
    {
        if ($_POST['customer_identification']) {
            update_post_meta($orderId, '_customer_identification', esc_attr( $_POST['customer_identification']));
        }
    }

    public function displayCustomerIdentificationOnOrderThankyou($orderId)
    {
        if ( get_post_meta( $orderId, '_customer_identification', true ) ) {
            echo '<p><strong>Customer identification:</strong> ' . get_post_meta( $orderId, '_customer_identification', true ) . '</p>';
        }
    }



    /**
     * Display the selected dynamic options on the cart page
     */
    public function displayCartItemDynamicOptions($itemData, $cartItem)
    {
        $cartContainStockProducts = false;
        $cartContainPrintProducts = false;
        foreach (WC()->cart->get_cart() as $cart_item_key => $cart_item) {
            if (!CloudPrinterHelper::isCloudPrinterProduct($cart_item['product_id'])) {
                $cartContainStockProducts = true;
            } else {
                $cartContainPrintProducts = true;
            }

            if ($cartContainStockProducts && $cartContainPrintProducts) {
                break;
            }
        }

        $viewData = [
            'isCloudPrinterProduct' => CloudPrinterHelper::isCloudPrinterProduct($cartItem['product_id']),
            'cartContainStockProducts' => $cartContainStockProducts,
            'cartContainPrintProducts' => $cartContainPrintProducts,
        ];

        if (!empty($cartItem['dynamicOptions'])) {
            $viewData['productDynamicOptions'] = CloudPrinterHelper::getItemDynamicOption($cartItem['product_id'], unserialize($cartItem['dynamicOptions']));
        }

        echo $this->getHtml('product-type', $viewData);

        return $itemData;
    }

    /**
     * Clear the shipping quotes after add/remove product in cart.
     */
    public function clearShippingQuotes()
    {
        $wcSession = CloudPrinterHelper::getWCSession();
        $wcSession->set('cp_shipping_data', '');
    }

    /**
     * Custom validation for checkout form
     */
    public function validationCheckoutForm($fields, $errors)
    {
        if (strlen($fields['billing_address_1']) > 30) {
            $errors->add('validation', 'The street address shouldn`t be more then 30 chars.');
        }

        if (strlen($fields['billing_address_2']) > 30) {
            $errors->add('validation', 'The additional address shouldn`t be more then 30 chars.');
        }
    }

    /**
     * Store the order item metadata
     */
    public function checkoutCreateOrderLineItem($item, $cart_item_key, $values, $order)
    {
        $item->add_meta_data('_cartItemKey', $cart_item_key);
    }

    /**
     * Make the shipping address required.
     */
    public function needShippingAddress()
    {
        return true;
    }

    /**
     * Save custom checkout fields
     * @param $orderId
     */
    public function saveCustomCheckoutFields($orderId)
    {
        $orderModel = new \WC_Order($orderId);
        $orderItems = $orderModel->get_items();
        $dynamicProductService = new DynamicProductService();

        // prepare meta data for dynamic product.
        /** @var \WC_Order_Item_Product $orderItem */
        $dynamicProductFiles = [];
        $dynamicProductFilesNames = [];

        foreach ($orderItems as $orderItem) {
            $productId = $orderItem->get_product_id();
            if (CloudPrinterHelper::isCloudPrinterProduct($productId)) {
                $productModel = new CPProduct($productId);
                $cartItemKey = $orderItem->get_meta('_cartItemKey');
                if ($productModel->isDynamicProduct()) {
                    $dynamicProductFiles[$cartItemKey] = [];
                    $productFilesInfo = $productModel->getCPProductFilesInfo();
                    $storage = $_POST['cp_file_storage'][$cartItemKey];
                    foreach ($productFilesInfo as $file) {
                        $fileName = "";
                        switch ($storage) {
                            case 'local':
                                $fileName = $_POST['cp_dynamic_file_name'][$cartItemKey][$file['type']];
                                $dynamicProductFilesNames[] = $fileName;
                                break;
                            case 'external':
                                $fileName = $_POST['cp_file_url'][$cartItemKey][$file['type']];
                        }
                        $dynamicProductFiles[$cartItemKey][] = [
                            'storage' => $storage,
                            'file_storage_service' => $dynamicProductService->getStorageServiceName(),
                            'type' => $file['type'],
                            'file' => $fileName
                        ];


                        // Clear cash
                        delete_transient('user_uploaded_file_' . $cartItemKey . '_' . $file['type']);
                    }
                }
            }
        }

        $dynamicProductService = new DynamicProductService();
        $storageService = $dynamicProductService->getStorageService();

        // Move files from tmp folder
        if ($dynamicProductFilesNames) {
            try {
                $storageService->moveFilesFromTmpFolderToOrdered($dynamicProductFilesNames);
            } catch (\Exception $e) {
                LogHelper::warning('dynamic-files', $e->getMessage());
            }
            wp_clear_scheduled_hook('cp_clear_dynamic_product_storage', [$dynamicProductService->getStorageServiceName(), $dynamicProductFilesNames]);
        }

        // Save dynamic product meta fata
        if ($dynamicProductFiles) {
            update_post_meta($orderId, '_cp_dynamic_product', json_encode($dynamicProductFiles));
        }

        if (!empty($_POST['shipping_phone'])) {
            update_post_meta($orderId, '_shipping_phone', sanitize_text_field($_POST['shipping_phone']));
        }

        if (!empty($_POST['shipping_email'])) {
            update_post_meta($orderId, '_shipping_email', sanitize_text_field($_POST['shipping_email']));
        }
    }

    public function changeCountryLocale($countries)
    {
        $countriesWithoutZip = CountryInterface::COUNTRIES_WITHOUT_ZIP;

        foreach ($countries as $code => $countryData) {
            $countries[$code]['postcode']['required'] = !in_array($code, $countriesWithoutZip);
        }

        foreach ($countriesWithoutZip as $countryCodeWithoutZip) {
            if (!isset($countries[$countryCodeWithoutZip])) {
                $countries[$countryCodeWithoutZip]['postcode']['required'] = false;
            }
        }

        return $countries;
    }

    /**
     * Make the phone number required for the checkout page.
     */
    public function overrideCheckoutFields($fields)
    {
        if (isset($fields['billing']['billing_phone'])) {
            $fields['billing']['billing_phone']['required'] = true;
        }

        $fields['shipping']['shipping_phone'] = [
            'label' => 'Phone',
            'required' => true,
            'type' => 'tel',
            'class' => ['form-row-wide'],
            'validate' => ['phone'],
            'priority' => 100
        ];

        $fields['shipping']['shipping_email'] = [
            'label' => 'Email address',
            'required' => true,
            'type' => 'email',
            'class' => ['form-row-wide'],
            'validate' => ['email'],
            'priority' => 100
        ];

        return $fields;
    }

    /**
     * Api callback handler.
     * Store selected print shipping method. On cart page.
     * @param $postData
     * @return array
     */
    public function savePrintShipping($postData)
    {
        include_once WC_ABSPATH . 'includes/wc-cart-functions.php';
        include_once WC_ABSPATH . 'includes/wc-notice-functions.php';

        $wcSession = CloudPrinterHelper::getWCSession();

        if (!empty($postData['cp_shipping_data'])) {
            $postCpShippingData = $postData['cp_shipping_data'];
            $cpShippingData = wc_clean(wp_unslash(json_encode($postCpShippingData)));

            $wcSession->set('cp_shipping_data', $cpShippingData);
            //  $wcSession->set('cp_product_price', $postCpShippingData[0]['price']);
        }

        return ['success' => true];
    }

    /**
     * Display shipping for print products.
     */
    public function displayShippingForPrintProduct()
    {
        $shippingMarginPercentage = get_option('cp_shipping_price_margin');
        $shippingCountry = WC()->customer->get_shipping_country();
        $shippingPostcode = WC()->customer->get_shipping_postcode();
        if ($this->cartContainPrintProduct() && $shippingCountry) {
            $quote = $this->getOrderQuote();
            $wcSession = CloudPrinterHelper::getWCSession();
            if (isset($quote['error'])) {
                $shippingData = $wcSession->set('cp_shipping_data', '');
                $wcSession->set('cp_product_price', '');
            } else {
                $shippingData = $wcSession->get('cp_shipping_data');
                $shippingData = $shippingData ? json_decode($shippingData, true) : false;
            }
            $productNames = $this->getProductNamesFromCart();

            $this->render('cloudprinter-custom-shipping', [
                'quote' => $quote,
                'productNames' => $productNames,
                'shippingData' => $shippingData,
                'shippingMarginPercentage' => $shippingMarginPercentage,
                'isFreeShipping' => $this->isFreeShipping || CartHelper::cartHasFreeShippingCoupon(),
                'shippingCountry' => $shippingCountry,
                'shippingPostcode' => $shippingPostcode,
            ]);

            WC()->cart->calculate_totals();
        }
    }


    private function getProductNamesFromCart()
    {
        $result = [];

        foreach (WC()->cart->get_cart() as $key => $cart_item) {
            $result[$key] = $cart_item['data']->get_title();
        }

        return $result;
    }

    /**
     * Change label for stock shipping methods.
     * @return string
     */
    public function setShippingStockName($shippingLabel)
    {
        $productNames = $this->getProductNamesFromCart();
        $stockProductIds = $this->getStockProductIds();
        $stockProductsNames = array_intersect_key($productNames, array_flip($stockProductIds));

        if (count($stockProductsNames) === count($productNames)) {
            return $shippingLabel;
        }

        return 'Shipping for stock products <br><small>(' . join(', ', $stockProductsNames) . ')</small>';
    }

    /**
     * Enable stock shipping for stock products.
     * @return bool
     */
    public function stockShippingEnabled()
    {
        if (!$this->cartContainStocProduct()) {
            return false;
        }

        return true;
    }


    /**
     * Custom validation for checkout form.
     * @throws \Exception
     */
    public function checkoutValidate()
    {
        $orderQuote = $this->getOrderQuote();
        $apiKey = get_option('cp_api_key');

        if ($this->cartContainPrintProduct()) {
            if (!$apiKey) {
                throw  new \Exception('Checkout is not possible, please contact support');
            }

            if (!$_POST['cp_shipping_hash'] || count($_POST['cp_shipping_hash']) < count($orderQuote['shipments'])) {
                throw  new \Exception('Shipping is not selected for print products');
            }

            if (!count($orderQuote['shipments'])) {
                throw  new \Exception('No general available production in the region');
            }

            $wcSession = CloudPrinterHelper::getWCSession();
            $selectedShipping = $wcSession->get('cp_shipping_data');
            if (empty($selectedShipping)) {
                LogHelper::warning('warning-checkout-validation', 'The shipping is empty in the session. Order data: ' . print_r($_POST, true));
                throw  new \Exception('There was an error while processing the shipping. Please, try again later.');
            }
        }

        // Price from orders/quote
        $cartItems = WC()->cart->get_cart_contents();
        $totalPrintProductPrice = 0;
        $printProducts = [];
        foreach ($cartItems as $cartItemKey => $cartItem) {
            if (CloudPrinterHelper::isCloudPrinterProduct($cartItem['product_id'])) {
                $productModel = new CPProduct($cartItem['product_id']);
                $totalPrintProductPrice += $cartItem['line_subtotal'];

                $printProducts[] = [
                    'name' => $productModel->get_name(),
                    'price' => $cartItem['line_subtotal'],
                    'quantity' => $cartItem['quantity'],
                    'cartItem' => $cartItem
                ];

                // Validate files for dynamic file product
                if ($productModel->isDynamicProduct()) {
                    $productFilesInfo = $productModel->getCPProductFilesInfo();
                    $storage = $_POST['cp_file_storage'][$cartItemKey];
                    foreach ($productFilesInfo as $file) {
                        switch ($storage) {
                            case 'local':
                                if (!$_POST['cp_dynamic_file_name'][$cartItemKey][$file['type']]) {
                                    throw new \Exception("The " .$file['type']. " file is required for the '" . $productModel->get_name() ."'' product.");
                                }
                                break;
                            case 'external':
                                if (!$_POST['cp_file_url'][$cartItemKey][$file['type']]) {
                                    throw new \Exception("The " .$file['type']. " file url is required for the '" . $productModel->get_name() ."'' product.");
                                }
                        }
                    }
                }
            }
        }

        // Check revenue
        $wcSession = CloudPrinterHelper::getWCSession();
        $purchasePrice = $wcSession->get('cp_product_price');
        $configService = new ConfigService();
        $revenueCheckService = new RevenueCheckService();
        if (!$revenueCheckService->checkRevenue($totalPrintProductPrice, $purchasePrice, $printProducts)) {
            throw new \Exception('Price calculation on the product was not possible, please contact support - ' .  $configService->getSupportEmail());
        }
    }

    /**
     * Calculate total price for order.
     * @param $total
     * @return mixed
     * @throws \CloudPrinter\CloudApps\Exception\ValidationException
     */
    public function calculateTotalOrderPrice($total)
    {
        if ($this->cartContainPrintProduct()) {
            $wcSession = CloudPrinterHelper::getWCSession();
            $shippingData = $wcSession->get('cp_shipping_data');
            $shippingData = $shippingData ? json_decode($shippingData, true) : [];

            if (!empty($shippingData)) {
                $customer = WC()->customer;
                $shippingCountry = $customer->get_shipping_country();
                $shippingPostcode = $customer->get_shipping_postcode();

                if (!$this->isFreeShipping && !CartHelper::cartHasFreeShippingCoupon() && $shippingData) {
                    $quote = $this->getOrderQuote();

                    if (!empty($quote['price'])) {
                        $wcSession->set('cp_product_price', $quote['price']);

                        $shippingPrice = 0;

                        if (isset($quote['shipments'])) {
                            foreach ($quote['shipments'] as $key => $shipment) {
                                if (isset($shipment['quotes'])) {
                                    foreach ($shipment['quotes'] as $shipping) {
                                        if (!empty($shippingData[$key]['service']) && $shipping['service'] == $shippingData[$key]['service'] && $shipping['shipping_option'] == $shippingData[$key]['shipping_option']) {
                                            $shippingData[$key]['quote'] = $shipping['quote'];
                                            $shippingPrice += $shipping['price'];
                                            break;
                                        }
                                    }
                                }
                            }
                        }
                        $shippingPrice = apply_filters('cloudprinter_shipping_price', $shippingPrice, $shippingCountry, $shippingPostcode);

                        $shippingMarginPercentage = get_option('cp_shipping_price_margin');
                        $shippingPriceInclMargin = CloudPrinterHelper::calcPercentage($shippingPrice, $shippingMarginPercentage);
                        $total += $shippingPriceInclMargin;

                        if (wc_tax_enabled() && 'incl' !== get_option( 'woocommerce_tax_display_shop' )) {
                            $shippingService = new ShippingService();
                            $total += $shippingService->calculateTax($shippingPriceInclMargin);
                        }


                    }
                }
            }
        }
        return $total;
    }

    /**
     * Request order quote.
     * @return array|boolean
     * @throws \CloudPrinter\CloudApps\Exception\ValidationException
     */
    private function getOrderQuote()
    {
        $cpAccessToken = get_option('cp_api_key');

        $customer = WC()->customer;
        $cloudPrinterClient = new CloudAppsClient($cpAccessToken);
        $orderQuote = new OrderQuote();
        $orderQuote->setCurrency(get_woocommerce_currency());
        $orderQuote->setCountry($customer->get_shipping_country() ?: $customer->get_billing_country());
        $orderQuote->setState($customer->get_shipping_state() ?: $customer->get_billing_state());

        $cartItems = WC()->cart->get_cart_contents();

        foreach ($cartItems as $cartItem) {
            $productId = $cartItem['product_id'];
            $cpProduct = new CPProduct($productId);
            $productReference = get_post_meta($productId, 'cp_product_reference', true);

            if (CloudPrinterHelper::isCloudPrinterProduct($productId) && $productReference) {
                $productPeaces = CloudPrinterHelper::getPeacesForProduct($productId);
                $orderQuoteItem = new OrderQuoteItem();
                $orderQuoteItem->setCount($cartItem['quantity'] * $productPeaces);
                $orderQuoteItem->setReference($cartItem['key']);
                $orderQuoteItem->setProductReference($productReference);

                if ($cpProduct->hasDynamicOptions()) {
                    $options = unserialize($cartItem['dynamicOptions']);
                } else {
                    $options = get_post_meta($productId, 'cp_product_options', true);
                    $options = $options ? unserialize($options) : [];
                }


                if (!empty($options) && is_array($options)) {
                    foreach ($options as $key => $option_reference) {
                        $option = new Option();

                        if ($key == 'type_archive_total_files') {
                            $option->setOptionReference('archive_total_files');
                            $option->setCount((int)$option_reference);
                        } else {
                            $option->setOptionReference($option_reference);
                            $option->setCount(1);
                        }

                        $orderQuoteItem->addOption($option);
                    }
                }

                $productCategory = get_post_meta($productId, 'cp_product_category', true);
                if (CloudPrinterHelper::isBookProduct($productCategory)) {
                    $countPages = get_post_meta($productId, 'cp_book_count_pages', true);
                    if ($countPages) {
                        $option = new Option();
                        $option->setOptionReference('total_pages');
                        $option->setCount($countPages);
                        $orderQuoteItem->addOption($option);
                    }
                }

                $orderQuote->addItem($orderQuoteItem);
            }
        }

        try {
            $quoteData = $orderQuote->toArray();
        } catch (ValidationException $e) {
            return false;
        }

        // Get order quote from cache.
        $hashSum = md5($cpAccessToken . json_encode($quoteData));
        $data = OrderQuoteCacheHelper::get($hashSum);

        if (!$data) {
            try {
                $response = $cloudPrinterClient->order->quote($orderQuote);
            } catch (ValidationException $e) {
                return false;
            }

            $data = $response->getData();

            OrderQuoteCacheHelper::set($hashSum, $data);

            // Write log
            $message = 'order/quote - checkout | Request: ' . json_encode($orderQuote->toArray()) . PHP_EOL . 'Response:' . json_encode($data);
            LogHelper::debug('debug-api', $message);
        }

        // Exclude expensive shipping If free shipping is enabled
        if ($this->isFreeShipping && $this->costCheck) {
            $overIncome = 0;
            if (!isset($data['error'])) {
                foreach ($cartItems as $cartItem) {
                    if (CloudPrinterHelper::isCloudPrinterProduct($cartItem['product_id'])) {
                        $minimumProductMargin = CloudPrinterHelper::getMinProductMargin();
                        $priceWithMargin = CloudPrinterHelper::calcPercentage($data['price'], $minimumProductMargin);
                        $overIncome += $cartItem['data']->get_price() - $priceWithMargin;
                    }
                }
            }

            foreach ($data['shipments'] as &$shipment) {
                if (isset($shipment['quotes'])) {
                    $lessPrice = null;
                    $lessKey = null;
                    foreach ($shipment['quotes'] as $key => $shipping) {
                        if ($lessPrice == null) {
                            $lessPrice = $shipping['price'];
                            $lessKey = $key;
                        } else if ($lessPrice > $shipping['price']) {
                            $lessPrice = $shipping['price'];
                            $lessKey = $key;
                        }
                    }
                    $shipment['quotes'] = [$shipment['quotes'][$lessKey]];

                    foreach ($shipment['quotes'] as $key => $shipping) {
                        if ($shipping['price'] > $overIncome) {
                            unset($shipment['quotes'][$key]);
                        }
                    }
                }
            }
        }
        if (!empty($data['price'])) {
            $wcSession = CloudPrinterHelper::getWCSession();
            $wcSession->set('cp_product_price', $data['price']);
        }
        return $data;
    }

    private function cartContainPrintProduct()
    {
        $cartItems = WC()->cart->get_cart_contents();

        foreach ($cartItems as $cartItem) {
            if (CloudPrinterHelper::isCloudPrinterProduct($cartItem['product_id'])) {
                return true;
            }
        }

        return false;
    }

    private function cartContainStocProduct()
    {
        if (WC()->cart) {
            $cartItems = WC()->cart->get_cart_contents();

            foreach ($cartItems as $cartItem) {
                $product = wc_get_product( $cartItem['product_id'] );
                if (!CloudPrinterHelper::isCloudPrinterProduct($cartItem['product_id']) && !$product->is_virtual()) {
                    return true;
                }
            }

            return false;
        }

        return true;
    }

    private function getStockProductIds()
    {
        $stockProductIds = [];
        foreach (WC()->cart->get_cart() as $key => $item) {
            /** @var \WC_Product $product */
            $product = $item['data'];
            if (!CloudPrinterHelper::isCloudPrinterProduct($product->get_id())) {
                $stockProductIds[] = $key;
            }
        }

        return $stockProductIds;
    }
}
