<?php

namespace CloudPrinterPlugin\Controllers;

use CloudPrinterPlugin\Helpers\CloudPrinterHelper;
use CloudPrinterPlugin\Helpers\OrderHelper;
use CloudPrinterPlugin\Libs\BaseController;
use CloudPrinterPlugin\Models\CPOrder;
use CloudPrinterPlugin\Models\CPProduct;
use CloudPrinterPlugin\Services\DynamicProduct\DynamicProductService;
use CloudPrinterPlugin\Services\ShippingService;

/**
 * Class OrderAdminController
 *
 * @copyright   2019 by CloudPrinter
 * @author      Vasyl Kushniruk <vk@cloudprinter.com>
 */
class OrderAdminController extends BaseController
{
    public function __construct()
    {
        parent::__construct(CP_ABS_PATH, 'order-admin');

        // Add meta-box with CloudPrinter order info.
        add_action('add_meta_boxes', [ $this, 'cloudPrinterOrderMetaBox' ], 10, 2);

        // Display print shipping
        add_action('woocommerce_admin_order_items_after_shipping', [ $this, 'displayPrintShipping' ]);

        // Display Item status. That comes from cloudprinter.
        add_action('woocommerce_before_order_itemmeta', [ $this, 'displayItemAdditionInfo' ], 10, 3);

        // Add reorder form in footer.
        add_action('admin_footer', [ $this, 'addReorderForm' ]);

        // Custom shipping fields.
        add_action('woocommerce_admin_shipping_fields', [ $this, 'addShippingFieldsToOrderFields' ]);
        add_filter('woocommerce_get_order_address', [ $this, 'getShippingCustomFieldsFromOrder' ], 10, 3);

        // Set values for custom columns.
        add_action('manage_shop_order_posts_custom_column', [ $this, 'setValueForCustomColumns' ], 10, 2);

        // Calculate shipping total
        add_filter('woocommerce_order_get_shipping_total', [ $this, 'includePrintShippingInTotalShippingPrice' ], 10, 2);

        // Exclude print products from stock shipping.
        add_filter('woocommerce_order_item_display_meta_value', [ $this, 'displayOnlyStockProduct' ], 10, 3);

        // Disable changing country and state on order edit page.
        add_filter('woocommerce_admin_shipping_fields', [ $this, 'adminShippingFields' ]);
        add_filter('woocommerce_admin_billing_fields', [ $this, 'adminBillingFields' ]);

        // Add custom columns to order list table.
        add_filter('manage_edit-shop_order_columns', [ $this, 'customOrderColumns' ]);

        // Search by order reference.
        add_filter("pre_get_posts", [ $this, "searchByCustomFields" ]);

        add_filter("woocommerce_hidden_order_itemmeta", [ $this, "hideOrderItemMeta" ]);

        // Forbid to add cp product to order from admin
        add_filter("woocommerce_json_search_found_products", [$this, 'blockAddingCloudprinterProductsToOrderFromAdmin']);

        // Display order identification in order admin
        add_action('woocommerce_admin_order_data_after_billing_address', [$this, 'displayCustomerIdentificationInAdmin']);
    }

    public function displayCustomerIdentificationInAdmin($order)
    {
        $orderId = $order->get_id();
        if (get_post_meta( $orderId, '_customer_identification', true )) {
            echo '<p><strong>Customer identification:</strong> ' . get_post_meta( $orderId, '_customer_identification', true ) . '</p>';
        }
    }

    public function hideOrderItemMeta($meta)
    {
        return array_merge($meta, [
            '_projectFileName',
            '_productDesignerTemplateName',
            '_cartItemKey',
            '_shipping_tracking_code',
            '_shipping_option',
            '_order_shipped_time',
            '_cp_item_status',
            '_shipping_tracking_url',
            '_reorder_cause',
            '_reorder_description',
            '_dynamicOptions'
        ]);
    }

    /**
     * Extend search in admin order page.
     * @param $query
     */
    public function searchByCustomFields($query)
    {
        $customFields = array(
            "_cp_order_reference",
        );
        $searchTerm = $query->query_vars['s'];
        $query->query_vars['s'] = "";

        if ($searchTerm != "") {
            $metaQuery = array('relation' => 'OR');
            foreach($customFields as $customField) {
                array_push($metaQuery, array(
                    'key' => $customField,
                    'value' => $searchTerm,
                    'compare' => 'LIKE'
                ));
            }
            $query->set("meta_query", $metaQuery);
        };
    }

    /**
     * @return mixed
     */
    public function addShippingFieldsToOrderFields($fields)
    {
        $fields['phone'] = ['label' => 'Phone'];
        $fields['email'] = ['label' => 'Email'];

        return $fields;
    }

    public function getShippingCustomFieldsFromOrder($fields, $address_type, \WC_Order $order)
    {
        $order_id = $order->get_id();
        if ('shipping' === $address_type) {
            $billingAddress = $order->get_address('billing');

            $shippingPhone = get_post_meta($order_id, '_shipping_phone', true);
            if (!$shippingPhone) {
                $shippingPhone = $billingAddress['phone'];
            }

            $shippingEmail = get_post_meta($order_id, '_shipping_email', true);
            if (!$shippingEmail) {
                $shippingEmail = $billingAddress['email'];
            }

            $fields['phone'] = $shippingPhone;
            $fields['email'] = $shippingEmail;
        }

        return $fields;
    }

    /**
     * Add custom columns.
     * @param $columns
     * @return array
     */
    public function customOrderColumns($columns)
    {
        return array_slice( $columns, 0, 4, true )
            + ['cp_order_status' => 'CP status']
            + ['cp_order_reference' => 'CP order reference']
            + array_slice( $columns, 4, NULL, true );
    }

    /**
     * Set values for custom columns.
     * @param $column
     * @param $orderId
     */
    public function setValueForCustomColumns($column, $orderId)
    {
        switch ($column) {
            case 'cp_order_status':
                $cpOrder = new CPOrder($orderId);
                //$cpOrder->get_shipping_total();
                $this->render('order-column-status', ['cpOrder' => $cpOrder]);
                break;
            case 'cp_order_reference':
                $cpOrder = new CPOrder($orderId);
                $this->render('order-column-reference', ['cpOrder' => $cpOrder]);
                break;
        }
    }

    /**
     * Disable edit billing country and state
     * @param array $fields
     * @return array
     */
    public function adminBillingFields($fields)
    {
        global $post;

        if (!empty($post) && $post->post_type == 'shop_order') {
            $order = new \WC_Order($post->ID);

            if (!$order->get_shipping_country()) {
                $fields['state']['custom_attributes'] = ['disabled' => true];
                $fields['country']['custom_attributes'] = ['disabled' => true];
            }
        }

        return $fields;
    }

    /**
     * Disable edit shipping country and state
     * @param array $fields
     * @return array
     */
    public function adminShippingFields($fields)
    {
        $fields['state']['custom_attributes'] = ['disabled' => true];
        $fields['country']['custom_attributes'] = ['disabled' => true];

       return $fields;
    }

    public function addReorderForm() {
        global $post;

        if (!empty($post) && $post->ID && $post->post_type == 'shop_order') {
            $order = new \WC_Order($post->ID);
            if ($order) {
                $this->render('reorder-form', ['order' => $order]);
            }
        }
    }

    /**
     * Display info about cloudpinter order.
     * @param $type
     * @param $post
     */
    public function cloudPrinterOrderMetaBox(string $type, $post)
    {
        if ($type == 'shop_order') {
            $order = wc_get_order($post->ID);

            if ($this->hasPrintProduct($order->get_items())) {
                add_meta_box(
                    'edit-shop_order',
                    __('Cloudprinter.com order info'),
                    [$this, 'render_my_meta_box'],
                    'shop_order',
                    'normal',
                    'high',
                    [$order]
                );
            }
        }
    }

    /**
     * Display item status info that comes from cloudprinter.
     * @param $itemId
     * @param $item
     * @param $product
     * @return bool
     */
    public function displayItemAdditionInfo(int $itemId, $item, $product)
    {
        if (!isset($product->id)) {
            return false;
        }

        $productId = $product->get_id();


        $productModel = new CPProduct($productId);

        // Escape if it is not cloudprinter product.
        if (get_post_meta($productId, '_cp_is_cloud_printer_product', true) != 'yes') {
            return false;
        }

        $itemStatus = wc_get_order_item_meta($itemId, '_cp_item_status');

        // Use productId to support old version of plugin.
        $cpItemReference = !empty($item['cartItemKey']) ? $item['cartItemKey'] : $productId;

        if (!empty($itemStatus)) {
            switch ($itemStatus) {
                case 'ItemCanceled':
                    $itemsCanceled = get_post_meta($item['order_id'], '_cp_items_canceled', true);
                    $itemsCanceled = $itemsCanceled ? json_decode($itemsCanceled, true) : null;
                    $this->render('order-item-canceled', [
                        'message' => $itemsCanceled[$cpItemReference]['message']
                    ]);
                    break;
                case 'ItemError':
                    $itemsError = get_post_meta($item['order_id'], '_cp_item_error', true);
                    $itemsError = $itemsError ? json_decode($itemsError, true) : null;
                    $this->render('order-item-error', [
                        'message' => $itemsError[$cpItemReference]['message'],
                        'delay' => $itemsError[$cpItemReference]['delay'],
                    ]);
                    break;
                case 'ItemValidated':
                    $this->render('order-item-validated');
                    break;
                case 'ItemShipped':
                    $this->render('order-item-shipped', [
                        'trackingNumber' => wc_get_order_item_meta($itemId, '_shipping_tracking_code'),
                        'shippingService' => wc_get_order_item_meta($itemId, '_shipping_option'),
                    ]);
                    break;
            }
        }

        if ($productModel->isDynamicProduct()) {
            $dynamicProductService = new DynamicProductService();
            $storageServiceName = $dynamicProductService->getStorageServiceName();
            $dynamicProductData = json_decode(get_post_meta($item['order_id'],'_cp_dynamic_product', true), true);

            if (isset($dynamicProductData[$item['cartItemKey']])) {
                $this->render('dynamic-product-link', [
                    'dynamicFiles' => $dynamicProductData[$item['cartItemKey']],
                    'orderId' => $item['order_id'],
                    'storageServiceName' => $storageServiceName
                ]);
            }
        }

        $dynamicOptions = $item->get_meta('_dynamicOptions');
        if (!empty($dynamicOptions)) {
            $viewData = ['productDynamicOptions' => CloudPrinterHelper::getItemDynamicOption($item['product_id'], unserialize($dynamicOptions))];
            $this->render('item-meta', $viewData);
        }
    }

    public function render_my_meta_box($post, $meta)
    {
        $order = $meta['args'][0];
        $this->render('order-meta-box', ['order' => $order]);
        $this->render('post-again-form', ['order' => $order]);
    }

    /**
     * Exclude print products from stock shipping.
     * @param $metaValue
     * @param $meta
     * @param $item
     * @return string
     */
    public function displayOnlyStockProduct($metaValue, $meta, $item)
    {
        $result = [];
        $order = wc_get_order($item['order_id']);
        $products = $order->get_items();

        foreach ($products as $product) {
            if (!CloudPrinterHelper::isCloudPrinterProduct($product['product_id'])) {
                $result[] = $product['name'] . ' x ' . $product['quantity'];
            }
        }

        return join(', ', $result);
    }

    /**
     * @param $orderId
     */
    public function displayPrintShipping($orderId)
    {
        $order = wc_get_order($orderId);
        $products = $order->get_items();
        $shippingData = $order->get_meta('_cp_shipping_data');
        $isFreeShipping = $order->get_meta('_cp_shipping_is_free') || OrderHelper::hasFreeShippingCoupon($order);
        $shippingMarginPercentage = $order->get_meta('_cp_shipping_margin_percentage') ?: 0;

        $shippingData = $shippingData ? json_decode($shippingData, true) : [];

        $shippingRefundAmount = $order->get_meta('_shipping_refund_amount');
        $shippingRefundTax = $order->get_meta('_shipping_refund_tax');

        $result = [];
        foreach ($products as $product) {
            $cpItemReference = !empty($product['cartItemKey']) ? $product['cartItemKey'] : $product['product_id'];
            if (CloudPrinterHelper::isCloudPrinterProduct($product['product_id'])) {
                $result[$cpItemReference] = $product['name'] . ' x ' . $product['quantity'];
            }
        }


        $this->render('order-print-shipping', [
            'items' => $result,
            'shippingData' => $shippingData,
            'shippingMarginPercentage' => $shippingMarginPercentage,
            'isFreeShipping' => $isFreeShipping,
            'shippingRefundAmount' => $shippingRefundAmount,
            'shippingRefundTax' => $shippingRefundTax,
            'shippingService' => new ShippingService(),
            'order' => $order
        ]);
    }

    /**
     * In admin order page total shipping price calculated as price of stock shipping.
     * Here we include also print shipping.
     * @param $value
     * @param $post
     * @return float|int
     */
    public function includePrintShippingInTotalShippingPrice($value, $order)
    {
        if (is_admin()) {
            $isReorderedOrder = $order->get_meta('_reorder_origin_order_id');

            if (!$isReorderedOrder) {
                $shippingMarginPercentage = get_option('cp_shipping_price_margin');

                $shippingData = $order->get_meta('_cp_shipping_data');
                $shippingData = $shippingData ? json_decode($shippingData, true) : [];
                $shippingPriceInclMargin = 0;

                foreach ($shippingData as $shipping) {
                    if ($shipping['quote']) {
                        $orderQuote = CloudPrinterHelper::getOrderQuoteInfo($shipping['quote']);
                        $shippingCountry = $order->get_shipping_country();
                        $shippingPostcode = $order->get_shipping_postcode();
                        $shippingPrice = apply_filters('cloudprinter_shipping_price', $orderQuote['price'], $shippingCountry, $shippingPostcode);
                        $shippingPriceInclMargin += CloudPrinterHelper::calcPercentage($shippingPrice, $shippingMarginPercentage);
                    }
                }

                return $value + $shippingPriceInclMargin;
            }
        }

        return $value;
    }

    /**
     * @param $cartItems
     * @return bool
     */
    private function hasPrintProduct($cartItems)
    {
        foreach ($cartItems as $key => $cartItem) {
            if (CloudPrinterHelper::isCloudPrinterProduct($cartItem['product_id'])) {
                return true;
            }
        }

        return false;
    }

    public function blockAddingCloudprinterProductsToOrderFromAdmin($products)
    {
        if ($_GET['exclude_type'] === 'variable') {
            $printProductsIds = CloudPrinterHelper::getPrintProductsIds();

            foreach ($products as $productId => &$product) {
                if (in_array($productId, $printProductsIds)) {
                    unset($products[$productId]);
                }
            }
        }

        return $products;
    }
}
