<?php

namespace CloudPrinterPlugin\Controllers;

use CloudPrinterPlugin\Helpers\LogHelper;
use CloudPrinterPlugin\Libs\BaseController;
use CloudPrinterPlugin\Services\Storage\DropboxService;

/**
 * Class PluginValidateController
 *
 * @copyright   2019 by CloudPrinter
 * @author      Vasyl Kushniruk <vk@cloudprinter.com>
 */
class PluginValidateController extends BaseController
{
    const MIN_WOO_COMMERCE_VERSION = '4.9.2';
    const MIN_PHP_VERSION = '7.0';

    /**
     * PluginValidateController constructor.
     */
    public function __construct()
    {
        parent::__construct(CP_ABS_PATH, 'plugin-dependency');
        add_action('admin_init', [$this, 'validatePlugin']);
    }

    public function validatePlugin()
    {
        if (is_admin() && current_user_can('activate_plugins') && !is_plugin_active('woocommerce/woocommerce.php')) {
            add_action('admin_notices', [$this, 'displayWooCommerceRequireMessage']);
        } else if (!$this->checkWooCommerceVersion(self::MIN_WOO_COMMERCE_VERSION)) {
            add_action('admin_notices', [$this, 'displayNotValidWooCommerceVersionMessage']);
        }

        if (!version_compare(PHP_VERSION, self::MIN_PHP_VERSION, '>=')) {
            add_action('admin_notices', [$this, 'displayNotValidPhpVersionMessage']);
        }

        $apiKey = get_option('cp_api_key');
        if (!$apiKey) {
            add_action('admin_notices', [$this, 'displayNotAuthorizedMessage']);
        }

        $apiKey = get_option('cp_api_key');
        if (!$apiKey) {
            add_action('admin_notices', [$this, 'displayNotAuthorizedMessage']);
        }

        $dynamicProductStorage = get_option('cp_dynamic_product_storage');
        $dropboxAccessToken = get_option('cp_dropbox_access_token');
        if ($dynamicProductStorage == 'dropbox') {

            if (!$dropboxAccessToken) {
                add_action('admin_notices', [ $this, 'dropboxAccessTokenIsNotSet' ]);
            } else {
                $dropboxUsername = get_option('cp_dropbox_username');
                if ($dropboxAccessToken && !$dropboxUsername) {
                    add_action('admin_notices', [ $this, 'displayWrongDropboxAccessToken' ]);
                } else {
                    global $pagenow;
                    if ( $pagenow == 'index.php' ) {
                        $dropboxService = new DropboxService();
                        try {
                            $storageInfo = $dropboxService->getStorageInfo();
                            if ($storageInfo['allocation']['allocated'] - $storageInfo['used'] < (1024 * 1024 * 100)) {
                                add_action('admin_notices', [$this, 'dropboxStorageAlmostFull']);
                            }
                        } catch (\Exception $e) {
                            LogHelper::warning('dynamic-files', $e->getMessage());
                        }
                    }
                }
            }
        }
    }

    /**
     * @param $version
     * @return bool
     */
    private function checkWooCommerceVersion($version)
    {
        global $woocommerce;
            if (version_compare($woocommerce->version, $version, ">=")) {
                return true;
            }

        return false;
    }

    public function displayWooCommerceRequireMessage()
    {
        $this->render('warning-message', [
            'message' => 'Attention - The Cloudprinter.com plugin requires the WooCommerce plugin to be installed and active.'
        ]);
    }

    public function displayNotValidPhpVersionMessage()
    {
        $this->render('warning-message', [
            'message' => 'Attention -  The Cloudprinter.com plugin requires PHP version ' . self::MIN_PHP_VERSION . ' or higher. Your version' . PHP_VERSION . '.'
        ]);
    }

    public function displayNotValidWooCommerceVersionMessage()
    {
        global $woocommerce;

        $this->render('warning-message', [
            'message' => 'Attention - The Cloudprinter.com plugin requires WooCommerce version ' . self::MIN_WOO_COMMERCE_VERSION . ' or higher. Your version' . $woocommerce->version . '.'
        ]);
    }

    public function displayNotAuthorizedMessage()
    {
        $this->render('warning-message', [
            'message' => 'Attention - The Cloudprinter.com plugin requires a login. Go to the Cloudprinter.com plugin settings and log in.'
        ]);
    }

    public function displayWrongDropboxAccessToken()
    {
        $this->render('warning-message', [
            'message' => 'Attention - The Dropbox access token that was set in the Cloudprinter.com plugin settings is not valid.'
        ]);
    }

    public function dropboxAccessTokenIsNotSet()
    {
        $this->render('warning-message', [
            'message' => 'Attention - The Dropbox access token is not set in the Cloudprinter.com plugin settings.'
        ]);
    }

    public function dropboxStorageAlmostFull()
    {
        $this->render('warning-message', [
            'message' => 'Attention - The Dropbox storage is almost full'
        ]);
    }
}
