<?php

namespace CloudPrinterPlugin\Helpers;

use CloudPrinter\CloudApps\Client\CloudAppsClient;
use CloudPrinter\CloudApps\Model\Option;
use CloudPrinter\CloudApps\Model\OrderQuote;
use CloudPrinter\CloudApps\Model\OrderQuoteItem;
use CloudPrinterPlugin\Models\CPProduct;

/**
 * Class CloudPrinterHelper
 *
 * @copyright   2019 by CloudPrinter
 * @author      Vasyl Kushniruk <vk@cloudprinter.com>
 */
class CloudPrinterHelper
{
    /**
     * @return CloudPrinterHelper
     */
    public static function getInstance()
    {
        return new self();
    }

    /**
     * @param $quoteHash
     * @return array|bool
     */
    public static function getOrderQuoteInfo(string $quoteHash)
    {
        $apiKey = get_option('cp_api_key');
        $cloudPrinterClient = new CloudAppsClient($apiKey);
        $response = $cloudPrinterClient->order->quoteInfo($quoteHash);
        if (in_array($response->getStatusCode(), [200, 201])) {
            return $response->getData();
        }

        // Write log
        $message = 'order/quote/info | Request:' . $quoteHash . PHP_EOL . 'Response:' . json_encode($response->getData());
        LogHelper::debug('debug-api', $message);

        return false;
    }

    /**
     * @return array|bool
     */
    public static function webHookSubscribe()
    {
        $url = rest_url() . 'cloudprinter/v1/cloud-signal/handle';
        $apiKey = get_option('cp_api_key');
        if ($apiKey) {
            $cloudPrinterClient = new CloudAppsClient($apiKey);
            $response = $cloudPrinterClient->webHook->subscribe($url);
            $responseData = $response->getData();

            // Write log
            $message = 'web-hook/subscribe | Request:' . $url . PHP_EOL . 'Response:' . json_encode($responseData);
            LogHelper::debug('debug-api', $message);

            if (!empty($responseData['apikey'])) {
                update_option('cp_signal_api_key', $responseData['apikey']);
                update_option('cp_signal_id', $responseData['id']);
            }

            return $responseData;
        }
    }

    /**
     * @param int $webHookId
     * @return array|bool
     */
    public static function webHookUnSubscribe(int $webHookId)
    {
        $apiKey = get_option('cp_api_key');
        if ($apiKey) {
            $cloudPrinterClient = new CloudAppsClient($apiKey);
            $response = $cloudPrinterClient->webHook->unSubscribe($webHookId);
            $responseData = $response->getData();

            // Write log
            $message = 'web-hook/un-subscribe | Request: ID ' . $webHookId . PHP_EOL . 'Response:' . json_encode($responseData);
            LogHelper::debug('debug-api', $message);

            return $responseData;
        }
    }

    public static function getCountries()
    {
        $countryList = [];
        $apiKey = get_option('cp_api_key');
        $cloudPrinterClient = new CloudAppsClient($apiKey);
        $response = $cloudPrinterClient->shipping->getCountries();
        if (in_array($response->getStatusCode(), [200, 201])) {
            $responseData = $response->getData();
            $countryList = $responseData['shipping_countries'];
        }

        return $countryList;
    }

    public static function getStates($countryReference)
    {
        $countryList = [];
        $apiKey = get_option('cp_api_key');
        $cloudPrinterClient = new CloudAppsClient($apiKey);
        $response = $cloudPrinterClient->shipping->getStates($countryReference);
        if (in_array($response->getStatusCode(), [200, 201])) {
            $responseData = $response->getData();
            $countryList = $responseData['shipping_states'];
        }

        return $countryList;
    }

    public static function isBookProduct($category): bool
    {
        return in_array($category, ['Textbook BW', 'Textbook FC', 'Photobook', 'Brochure', 'Magazine']);
    }

    public static function isPhotoPrintProduct($category): bool
    {
        return $category === 'Photo print';
    }

    public static function getMinProductMargin()
    {
        return get_option('cp_min_product_price_margin', 0);
    }

    public static function isCloudPrinterProduct(int $productId)
    {
        return get_post_meta($productId, '_cp_is_cloud_printer_product', true) == 'yes';
    }

    public static function calcPercentage($price, $percentage)
    {
        return $price + ($price / 100 * $percentage);
    }

    public static function getPeacesForProduct($productId)
    {
        $productPeaces = get_post_meta($productId, 'cp_count_pieces', true);

        if (!$productPeaces || $productPeaces < 1) {
            $productPeaces = 1;
        }

        return $productPeaces;
    }

    public static function getCpProductInfo(string $productReference)
    {
        $accessToken = get_option('cp_api_key');
        $cloudPrinterClient = new CloudAppsClient($accessToken);
        $response = $cloudPrinterClient->product->getInfo($productReference);
        $responseData = $response->getData();

        return $responseData;
    }

    /**
     * @param \WC_Order $order
     * @param bool|array $onlyForItemIds
     * @param bool $itemsQuantity
     * @return array
     * @throws \CloudPrinter\CloudApps\Exception\ValidationException
     */
    public static function requestOrderQuotes(\WC_Order $order, $onlyForItemIds = [], $itemsQuantity = false)
    {
        $apiKey = get_option('cp_api_key');
        $printProducts = self::getPrintProducts($order);

        $cloudPrinterClient = new CloudAppsClient($apiKey);
        $orderQuote = new OrderQuote();
        $orderQuote->setCountry($order->get_shipping_country() ?: $order->get_billing_country());
        $orderQuote->setCurrency(get_woocommerce_currency());
        $orderQuote->setState($order->get_shipping_state() ?: $order->get_billing_state());
        $orderQuote->setState($order->get_shipping_state() ?: $order->get_billing_state());

        foreach ($printProducts as $product) {
            if ($onlyForItemIds && !in_array($product['item_key'], $onlyForItemIds)) {
                continue;
            } else if ($itemsQuantity && !empty($itemsQuantity[$product['item_key']])) {
                $product['product_quantity'] = $itemsQuantity[$product['item_key']];
            }

            $productPeaces = CloudPrinterHelper::getPeacesForProduct($product['product_id']);
            $orderQuoteItem = new OrderQuoteItem();
            $orderQuoteItem->setCount($product['product_quantity'] * $productPeaces);
            $orderQuoteItem->setReference($product['item_key']);
            $orderQuoteItem->setProductReference(get_post_meta($product['product_id'], 'cp_product_reference', true));

            foreach ($product['product_options'] as $key => $option_reference) {
                $option = new Option();

                if ($key == 'type_archive_total_files') {
                    $option->setOptionReference('archive_total_files');
                    $option->setCount($option_reference);
                } else {
                    $option->setOptionReference($option_reference);
                    $option->setCount(1);
                }

                $orderQuoteItem->addOption($option);
            }

            if (CloudPrinterHelper::isBookProduct($product['product_category'])) {
                $option = new Option();
                $option->setOptionReference('total_pages');
                $option->setCount(get_post_meta($product['product_id'], 'cp_book_count_pages', true));
                $orderQuoteItem->addOption($option);
            }

            $orderQuote->addItem($orderQuoteItem);
        }

        $response = $cloudPrinterClient->order->quote($orderQuote);
        $responseData = $response->getData();

        // Write log
        $message = 'order/quote | Request: ' . json_encode($orderQuote->toArray()) . PHP_EOL . 'Response:' . json_encode($responseData);
        LogHelper::debug('debug-api', $message);

        return $responseData;
    }

    /**
     * Get products with data that related to Cloudprinter
     * @param $order
     * @return array
     * @throws \Exception
     */
    public static function getPrintProducts(\WC_Order $order)
    {
        $products = [];
        $orderItems = $order->get_items();
        foreach ($orderItems as $itemId => $orderItem) {
            if (CloudPrinterHelper::isCloudPrinterProduct($orderItem['product_id'])) {
                $cpProduct = new CPProduct($orderItem['product_id']);

                if ($cpProduct->hasDynamicOptions()) {
                    $options = wc_get_order_item_meta($itemId, '_dynamicOptions');
                } else {
                    $options = get_post_meta($orderItem['product_id'], 'cp_product_options', true);
                }

                $options = $options ? unserialize($options) : [];
                $product = [
                    'product_reference' => get_post_meta($orderItem['product_id'], 'cp_product_reference', true),
                    'product_category' => get_post_meta($orderItem['product_id'], 'cp_product_category', true),
                    'product_id' => $orderItem['product_id'],
                    'product_quantity' => $orderItem['quantity'],
                    'product_options' => $options,
                    'item_key' => wc_get_order_item_meta($itemId, '_cartItemKey'),
                    'reorder_cause' => wc_get_order_item_meta($itemId, '_reorder_cause'),
                    'reorder_description' => wc_get_order_item_meta($itemId, '_reorder_description'),
                    'orderItemId' => $itemId,
                    'lineItem' => $orderItem,
                    'filesInfo' => $cpProduct->getCPProductFilesInfo()
                ];

                $files = [];

                if ($cpProduct->isStaticFileProduct()) {
                    $productFileStorage = get_post_meta($orderItem['product_id'], 'cp_file_storage', true);

                    $requiredProductFilesTypes = $cpProduct->getRequiredFilesTypes() ?: [];
                    $productFiles = $cpProduct->getProductFiles();

                    if ($productFileStorage == 'local') {
                        foreach ($requiredProductFilesTypes as $filesType) {
                            $files[$filesType] = [
                                'url' => $productFiles['local'][$filesType]['url'] ?: '',
                                'md5' => $productFiles['local'][$filesType]['md5'] ?: '',
                            ];
                        }
                    } else {
                        foreach ($requiredProductFilesTypes as $filesType) {
                            $files[$filesType] = [
                                'url' => $productFiles['external'][$filesType]['link'] ?: '',
                                'md5' => $productFiles['external'][$filesType]['md5'] ?: '',
                            ];
                        }
                    }
                }
                $product['files'] = $files;
                $products[] = $product;
            }
        }

        return $products;
    }

    public static function getProductNameById($items, $productId)
    {
        $productName = null;

        foreach ($items as $item) {
            /** @var \WC_Product $product */
            $product = $item->get_product();
            if ($product->get_id() == $productId) {
                $productName = $product->get_name();
                break;
            }
        }

        return $productName;
    }

    /**
     * Get info about authorized user.
     * @return array
     */
    public static function getUserInfo()
    {
        $apiKey = get_option('cp_api_key');
        $cloudPrinterClient = new CloudAppsClient($apiKey);
        $response = $cloudPrinterClient->user->getInfo();

        return $response->getData();
    }

    public static function isPluginActive( $plugin ) {
        return in_array( $plugin, (array) get_option( 'active_plugins', array() ), true ) || is_plugin_active_for_network( $plugin );
    }

    public static function getWCSession()
    {
        if (null === WC()->session) {
            $session_class = apply_filters('woocommerce_session_handler', 'WC_Session_Handler');
            WC()->session = new $session_class();
            WC()->session->init();
        }

        return WC()->session;
    }

    public static function getItemDynamicOption($productId, $itemSelectedDynamicOptions)
    {
        $result = [];
        $cpProduct = new CPProduct($productId);
        if ($cpProduct->hasDynamicOptions())
        {
            $productDynamicOptions = $cpProduct->getDynamicOptions();

            foreach ($productDynamicOptions as $productDynamicOption) {
                if (count($productDynamicOption['option_values']) > 1) {
                    $optionValueReference = $itemSelectedDynamicOptions[$productDynamicOption['option_reference']];
                    $value = null;
                    foreach ($productDynamicOption['option_values'] as $optionValue) {
                        if($optionValue['value'] == $optionValueReference) {
                            $value = $optionValue['name'];
                        }
                    }

                    $result[] = [
                        'name' => $productDynamicOption['option_name'],
                        'value' => $value
                    ];
                }
            }
        }

        return $result;
    }

    public static function getPrintProductsIds()
    {
        global $wpdb;

        $result = $wpdb->get_col( $wpdb->prepare( "
            SELECT a.id FROM {$wpdb->prefix}posts a, {$wpdb->prefix}postmeta b
            WHERE a.ID = b.post_id 
            AND a.post_type = 'product'
            AND b.meta_key = '_cp_is_cloud_printer_product'
            AND b.meta_value = 'yes'
        "));

        return $result;
    }
}
